<?php
# Block direct access to this file
if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

/**********************************************************************
 *  OnlineNIC - SSL WHMCS module
 *
 * Copyright (C) 2018 by Tucows Inc./OpenSRS.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 **********************************************************************/

# Set required files
require_once dirname(__FILE__).'/src/controller/controller.OnlineNICSSL.php';
require_once dirname(__FILE__).DS.'src'.DS.'model'.DS.'model.OnlineNICSSL.php';
//
// Old module file requirement
require_once dirname(__FILE__).DS.'core'.DS.'openSRS.php';

# Set Laravel class alias
use Illuminate\Database\Capsule\Manager as Capsule;

# Set alias to Database access class
use DataAccess\DataAccessOln as DataAccess;

// Set Global variables (Deprecated)
$onlinenic_ssl_language = array();

/**************************** PRIMARY FUNCTIONS ************************************/

/**
 * Module Config
 * @return array $configarray::Module config settings
 */
function onlinenic_ssl_ConfigOptions() {
    # Add custom ssl config email template
    DataAccess::CreateEmailTemplate();

    # Set config array
    $configarray = array(
        'username' =>  array(
            'FriendlyName'  =>  'OnlineNIC Reseller ID',
            'Type'          =>  'text',
            'Size'          =>  '25'
        ),
        'live_api_key' => array(
            'FriendlyName'  =>  'Live API Key',
            'Type'          =>  'text',
            'Size'          =>  '25'
        ),
        'cert_type' => array(
            'FriendlyName'  =>  'Certificate Type',
            'Type'          =>  'dropdown',
            'Options'       =>  implode(',', array_keys(onlinenic_ssl_getCertType()))
        ),
        'test_api_key' => array(
            'FriendlyName'  =>  'Test API Key',
            'Type'          =>  'text',
            'Size'          =>  '25'
        ),
        'test_mode' => array(
            'FriendlyName'  =>  'Test Mode',
            'Type'          =>  'yesno',
        ),
        'period' => array(
            'FriendlyName'  =>  'Default Registration Period',
            'Type'          =>  'text',
            "Default"       =>  '1',
            "Description"   =>  'Set to 1'
        ),
        'password' => array(
            'FriendlyName'  =>  'Password',
            'Type'          =>  'text',
            'Size'          =>  '25'
        ),
        'server_count' => array(
            'FriendlyName'  =>  'Server license',
            'Type'          =>  'text',
            'Default'       =>  '1',
            'Description'   =>  'Unlimited. Set to 1'
        ),
    );

    # Return config array
    return $configarray;
}

/**
 * Create Account
 * @param array $params::Data from WHMCS
 * @return mixed|string::Operation status or error message
 */
function onlinenic_ssl_CreateAccount($params) {
    # Create new controller class
    $sslController = new ControllerOnlineNICSSL($params);

    # Set global variables
    GLOBAL $CONFIG;

    # Get SSL order
    $order_detail = DataAccess::GetSSLProduct($params['serviceid']);

    # GC::Check existing cert order
    if($order_detail->remoteid) {
        # Return error
        return onlinenic_ssl_translate('certificate_already_exists');
    }

    if (empty($order_detail)) {
        # Set default registration period to 1
        $period = $params['configoption6'];
        # Set seal in search option for Symantec SSL products
        $seal_in_search = $params['configoption7'];
        # Set default server usage count. This is now unlimited by all vendors. Default to 1
        $server_count = $params['configoption8'];
        # Set server type
        $type = $params['configoption3'];

        # Construct order settings
        $details = array(
            'type' => $type,
            'period' => $period,
            'seal_in_search' => $seal_in_search,
            'server_count' => $server_count
        );

        # Reset username/password to blank in table "tblhosting"
        DataAccess::ResetUserInfo('', '', $params['serviceid']);

        # Create new order and get database ID
        $id = DataAccess::CreateOrder($params);

        # Construct final SSL product config link
        $url = '<a href="'.$CONFIG["SystemURL"].'/configuressl.php?cert='.md5($id).'">'.$CONFIG["SystemURL"].'/configuressl.php?cert='.md5($id).'</a>';

        # Send email template "OpenSRS - SSL Certificate Configuration Required" to client
        # WHMCS FUNCTION::sendMessage(string $template_name, int $service_id, array $template variables)
        sendMessage("OpenSRS - SSL Certificate Configuration Required", $params["serviceid"], array("ssl_configuration_link" => $url));

        # Module log
        $request_string = array(
            'details' => $details,
            'verify_link' => $CONFIG["SystemURL"].'/configuressl.php?cert='.md5($id),
            'params' => $params
        );

        # Set operations status for log
        $status = (!empty($id) ? $status = 'Success' : $status = 'Error: Failed to create order in local database.');

        # Module log
        $sslController->ModuleLog(__FUNCTION__, $request_string, $status, '', $params);

        # Return status
        return 'success';
    } else {
        return 'Order already created. Please configure this product instead to activate it.';
    }

}

/**
 * Admin Services Tab Fields
 * @param array $params::Data from WHMCS
 * @return array::List of field titles and values
 */
function onlinenic_ssl_AdminServicesTabFields($params) {
    # Create smarty instance
    $smartyClass = new Smarty();

    # Set template directory
    $smartyClass->setTemplateDir(dirname(__FILE__).'/src/view/');

    # GC::Check if order exists
    $orderId = $params['model']['attributes']['orderid'];
    $service_data = DataAccess::GetSSLProduct($orderId);

    # GC::Check if SSL service exits
    if(isset($service_data)) {
        # Get config data
        $config_data = json_decode($service_data->configdata);
        # Construct SSL server types drop down menu
        $server_types = onlinenic_ssl_whmcsServerTypes();
        $select_server = '';
        foreach($server_types as $key => $value) {
            $select_server .= '
            <option '.($key == $config_data->server_type ? 'selected="selected"' : '' ).' value="'.$key.'">'.$value.'</option>
            ';
        }

        # GC::Check if OpenSRS ID exists
        if($service_data->remoteid) {
            # Set order ID
            $up_order_id = $service_data->remoteid;

            //Get order info from OnlineNIC
            if($params['configoption5'] == 'on'){
                $username = 135610;
                $password = 654123;
                $api_key = $params['configoption4'];
                $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
            }else{
                $username = $params['configoption1'];
                $password = $params['configoption7'];
                $api_key = $params['configoption2'];
                $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
            }
            $timestamp = time();
            $token = md5($username.md5($password).$timestamp."getSSLOrderInfo");
            $data = array(
                'user'=>$username,
                'timestamp'=>$timestamp,
                'token'=>$token,
                'apikey'=>$api_key,
                'orderid'=>$up_order_id
            );
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            $res = curl_exec($curl);
            curl_close($curl);
            $res = json_decode($res);
            if($res->code != "1000"){
                return array('error' => onlinenic_ssl_translate($res->msg));
            }
            $order_data = $res->data;
            # Set page style
            $style = $smartyClass->fetch('style.AdminProductServicesTab.tpl');
            $product_type = onlinenic_ssl_getCertName($order_data->productid);
            $contact_email = $order_data->approvalemail;
            $approval_email = $order_data->approvalemail;
            $csr = $order_data->csr;
            $status = strtolower($order_data->status);
            $dv_method = isset($order_data->dcvmethod)?$order_data->dcvmethod:'email';

            # Set admin product/services tab variables
            $smartyClass->assign('style', $style);
            $smartyClass->assign('remote_id', $up_order_id);
            $smartyClass->assign('product_type', $product_type);
            $smartyClass->assign('contact_email', $contact_email);
            $smartyClass->assign('approval_email', $approval_email);
            $smartyClass->assign('csr', $csr);
            $smartyClass->assign('server_type', $select_server);

            # Initialize variable
            $product_details = array();

            switch ($status) {
                # Completed state
                case 'complete':
                    # Set expiry date
                    $expiry_date = date_format(date_create($order_data->expire), 'Y-m-d');

                    # Set order status
                    $order_status = '<span class="label label-success"> '. $status .' </span>';

                    # Set DV method
                    if(!empty($dv_method)) {
                        $dv_method = '<span class="label label-info"> '. $dv_method .' </span>';
                    } else {
                        $dv_method = '<span class="label label-warning"> N/A </span>';
                    }

                    # Set SSL cert
                    if($params['configoption5'] == 'on'){
                        $username = 135610;
                        $password = 654123;
                        $api_key = $params['configoption4'];
                        $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLCert";
                    }else{
                        $username = $params['configoption1'];
                        $password = $params['configoption7'];
                        $api_key = $params['configoption2'];
                        $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLCert";
                    }
                    $token = md5($username.md5($password).$timestamp."getSSLCert");
                    $data = array(
                        'user'=>$username,
                        'timestamp'=>$timestamp,
                        'token'=>$token,
                        'apikey'=>$api_key,
                        'orderid'=>$up_order_id
                    );
                    $curl = curl_init();
                    curl_setopt($curl, CURLOPT_URL, $url);
                    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
                    $res = curl_exec($curl);
                    curl_close($curl);
                    $res = trim($res);
                    $res = str_replace("\n","",$res);
                    $res = json_decode($res);
                    $cert = $res->cert;
                    # Set renewable date
                    $renewal_date =  date('Y-m-d', strtotime($expiry_date. '-59 days'));
                    if(strtotime($renewal_date) < strtotime(date("Y/m/d"))) {
                        $renewable_status =  '<span class="label label-success"> Renewable </span>';
                    } else {
                        $renewable_status =  '<span class="label label-warning"> Not renewable </span>';
                    }

                    # Update next due date
                    DataAccess::UpdateNextDueDate($service_data->serviceid, $expiry_date);

                    # Set admin product/services tab variables
                    $smartyClass->assign('order_status', $order_status);
                    $smartyClass->assign('dv_method', $dv_method);
                    $smartyClass->assign('renewable', $renewal_date .' | '. $renewable_status);
                    $smartyClass->assign('expiry_date', $expiry_date);
                    $smartyClass->assign('cert', $cert);

                    # Get admin product/services tab output
                    $ssl_details = $smartyClass->fetch('view.AdminProductServicesTab.tpl');

                    # Set product details for output
                    $product_details = array(
                        'OnlineNIC Details' => $ssl_details
                    );
                    break;

                # Approval required state
                case 'pending':
                    DataAccess::UpdateStatus($params['serviceid'], 'Configuration Complete');
                    # Set order status
                    $order_status = '<span class="label label-warning"> '. $status .' </span>';

                    # Set DV method
                    if(!empty($dv_method)) {
                        $dv_method = '<span class="label label-info"> '. $dv_method .' </span>';
                    } else {
                        $dv_method = '<span class="label label-warning"> N/A </span>';
                    }

                    # Set admin product/services tab variables
                    $smartyClass->assign('order_status', $order_status);
                    $smartyClass->assign('dv_method', $dv_method);
                    $smartyClass->assign('renewable', '<span class="label label-warning"> N/A </span>');
                    $smartyClass->assign('expiry_date', '<span class="label label-warning"> N/A </span>');

                    # Get admin product/services tab output
                    $ssl_details = $smartyClass->fetch('view.AdminProductServicesTab.tpl');

                    # Set product details for output
                    $product_details = array(
                        'OnlineNIC Details' => $ssl_details
                    );
                    break;

                # Declined state
                case 'cancelled':
                case 'refund':
                    # Mark SSL order as terminated
                    DataAccess::TerminateSSLOrder($params['serviceid']);

                    # Set order status
                    $order_status = '<span class="label label-success"> '. $status .' </span>';
                    # Set DV method
                    if(!empty($dv_method)) {
                        $dv_method = '<span class="label label-info"> '. $dv_method .' </span>';
                    } else {
                        $dv_method = '<span class="label label-warning"> N/A </span>';
                    }

                    # Set admin product/services tab variables
                    $smartyClass->assign('order_status', $order_status);
                    $smartyClass->assign('dv_method', $dv_method);
                    $smartyClass->assign('renewable', '<span class="label label-warning"> N/A </span>');
                    $smartyClass->assign('expiry_date', '<span class="label label-warning"> N/A </span>');

                    # Get admin product/services tab output
                    $ssl_details = $smartyClass->fetch('view.AdminProductServicesTab.tpl');

                    # Set product details for output
                    $product_details = array(
                        'OnlineNIC Details' => $ssl_details
                    );
                    break;

                # All other states
                default:
                    # Mark SSL order as terminated
                    DataAccess::TerminateSSLOrder($params['serviceid']);

                    # Set order status
                    $order_status = '<span class="label label-success"> ERROR::1110::No Order found </span>';

                    # Set DV method
                    if(!empty($dv_method)) {
                        $dv_method = '<span class="label label-info"> '. $dv_method .' </span>';
                    } else {
                        $dv_method = '<span class="label label-warning"> N/A </span>';
                    }

                    # Set admin product/services tab variables
                    $smartyClass->assign('order_status', $order_status);
                    $smartyClass->assign('dv_method', $dv_method);
                    $smartyClass->assign('renewable', '<span class="label label-warning"> N/A </span>');
                    $smartyClass->assign('expiry_date', '<span class="label label-warning"> N/A </span>');

                    # Get admin product/services tab output
                    $ssl_details = $smartyClass->fetch('view.AdminProductServicesTab.tpl');

                    # Set product details for output
                    $product_details = array(
                        'OnlineNIC Details' => $ssl_details
                    );
                    break;
            }

            # Return output
            return $product_details;
        } else {
            # Set error message field
            $error_msg = '<p> SSL order requires setup. </p>';

            $error_msg_details = '<span class="label lb-lg label-warning"> If you want to manage an existing SSL order, please enter the OpenSRS SSL order ID and select "Save Changes" button. </span>';

            # Set page style
            $style = $smartyClass->fetch('style.AdminProductServicesTab.tpl');

            # Enable input
            $disable_input = True;

            # Set template variables
            $smartyClass->assign('style', $style);
            $smartyClass->assign('error_msg', $error_msg);
            $smartyClass->assign('error_msg_details', $error_msg_details);
            $smartyClass->assign('disable_input', $disable_input);

            # Get error message template
            $output = $smartyClass->fetch('view.ErrorMessages.tpl');

            # Return output
            return array(
                'Error' => $output,
            );
        }
    } else {
        # Set error message field
        $error_msg = '<p> SSL order has not been created. Please select the "Create" button to start setup. </p>';

        # Set page style
        $style = $smartyClass->fetch('style.AdminProductServicesTab.tpl');

        # Disable input
        $disable_input = False;

        # Set template variables
        $smartyClass->assign('style', $style);
        $smartyClass->assign('error_msg', $error_msg);
        $smartyClass->assign('disable_input', $disable_input);

        # Get error message template
        $output = $smartyClass->fetch('view.ErrorMessages.tpl');

        # Return output
        return array(
            'Error' => $output,
        );
    }
}

/**
 * Admin Services Tab Fields Save
 * @param array $params::Data from WHMCS
 * @return string $results::SSL product details
 */
function onlinenic_ssl_AdminServicesTabFieldsSave($params) {
    # Handle order ID update
    if(!empty($_REQUEST['order_id'])) {
        $orderId = $params['model']['attributes']['orderid'];
        # Update database with new order ID
        DataAccess::UpdateOrder($orderId, 'remoteid', $_REQUEST['order_id']);

        # Get service ID data from DB
        $service_data = DataAccess::GetSSLProduct($orderId);

        # Get SSL order details from OnlineNIC
        if($params['configoption5'] == 'on'){
            $username = 135610;
            $password = 654123;
            $api_key = $params['configoption4'];
            $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }else{
            $username = $params['configoption1'];
            $password = $params['configoption7'];
            $api_key = $params['configoption2'];
            $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }
        $timestamp = time();
        $token = md5($username.md5($password).$timestamp."getSSLOrderInfo");
        $data = array(
            'user'=>$username,
            'timestamp'=>$timestamp,
            'token'=>$token,
            'apikey'=>$api_key,
            'orderid'=>$service_data->remoteid
        );
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $res = curl_exec($curl);
        curl_close($curl);
        $res = json_decode($res);
        $order_data = $res->data;
        $status = strtolower($order_data->status);
        $configParams = json_decode($service_data->configdata);

        # GC::Check order status
        if($status == "complete") {
            # Get cert data from OnlineNIC
            if($params['configoption5'] == 'on'){
                $username = 135610;
                $password = 654123;
                $api_key = $params['configoption4'];
                $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLCert";
            }else{
                $username = $params['configoption1'];
                $password = $params['configoption7'];
                $api_key = $params['configoption2'];
                $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLCert";
            }
            $token = md5($username.md5($password).$timestamp."getSSLCert");
            $data = array(
                'user'=>$username,
                'timestamp'=>$timestamp,
                'token'=>$token,
                'apikey'=>$api_key,
                'orderid'=>$service_data->remoteid
            );
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            $res2 = curl_exec($curl);
            curl_close($curl);
            $res2 = json_decode($res2);
            $cert = $res2->cert;
            $csr = $configParams->csr;
            $csr_data  = openssl_csr_get_subject($csr);
            $domain = $csr_data['CN'];
            $product_type = onlinenic_ssl_getCertName($order_data->productid);
            $server_type = $_REQUEST['server_type'];
            $expiry_date = date_format(date_create($order_data->expire), 'Y-m-d');
            $order_state = $status;
            $supplier_id = $service_data->remoteid;
            $contact_email = $order_data->approvalemail;
            $approval_email = $order_data->approvalemail;
            $contact_set = '';
            $dv_method = isset($order_data->dcvmethod)?$order_data->dcvmethod:'email';
            $ca_bundle[0] = '';
            $ca_bundle[1] = '';

            # Update DB with SSL data
            $config_data = array(
                'product_id' => $service_data->remoteid,
                'supplier_id' => $supplier_id,
                'product_type' => $product_type,
                'server_type' => $server_type,
                'domain' => $domain,
                'expiry_date' => $expiry_date,
                'order_state' => $order_state,
                'contact_email' => $contact_email,
                'approval_email' => $approval_email,
                'contact_set' => $contact_set,
                'dv_method' => $dv_method,
                'csr_data' => $csr_data,
                'csr' => $csr,
                'cert' => $cert,
                'ca_bundle_1' => $ca_bundle[0],
                'ca_bundle_2' => $ca_bundle[1]
            );

            # Update database
            DataAccess::UpdateConfigData($orderId, $config_data);

            # Update next due date
            DataAccess::UpdateNextDueDate($service_data->serviceid, $expiry_date);
        }
    }
}

/**
 * Client Area Display
 * @param array $params::Data from WHMCS
 * @return string::Client area output
 */
function onlinenic_ssl_ClientArea($params) {
    $params['orderid'] =$params['model']['attributes']['orderid'];
    $id = DataAccess::CreateOrder($params);

    # Set order ID
    $service_data = DataAccess::GetSSLProduct($id);

    # Set service order ID
    $service_id = $service_data->id;

    # Initialize variables
    $code = '';
    $code .= '<table style="width: 100%">';

    $configParams = json_decode($service_data->configdata);

    if (!empty($service_data->remoteid)) {
        # Get SSL order
        //Order from OnlineNIC
        if($params['configoption5'] == 'on'){
            $username = 135610;
            $password = 654123;
            $api_key = $params['configoption4'];
            $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }else{
            $username = $params['configoption1'];
            $password = $params['configoption7'];
            $api_key = $params['configoption2'];
            $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }
        $timestamp = time();
        $token = md5($username.md5($password).$timestamp."getSSLOrderInfo");
        $data = array(
            'user'=>$username,
            'timestamp'=>$timestamp,
            'token'=>$token,
            'apikey'=>$api_key,
            'orderid'=>$service_data->remoteid
        );
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $res = curl_exec($curl);
        curl_close($curl);
        $res = json_decode($res);
        if($res->code != "1000"){
            return array('error' => onlinenic_ssl_translate($res->msg));
        }
        $order_data = $res->data;

        # Set order details
        $status = strtolower($order_data->status);
        $subject  = openssl_csr_get_subject($configParams->csr);
        $domain = $subject['CN'];

        # GC::Check SSL order status
        switch ($status) {
            # Completed state
            case 'complete':
                # Get cert data from OnlineNIC
                if($params['configoption5'] == 'on'){
                    $username = 135610;
                    $password = 654123;
                    $api_key = $params['configoption4'];
                    $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLCert";
                }else{
                    $username = $params['configoption1'];
                    $password = $params['configoption7'];
                    $api_key = $params['configoption2'];
                    $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLCert";
                }
                $token = md5($username.md5($password).$timestamp."getSSLCert");
                $data = array(
                    'user'=>$username,
                    'timestamp'=>$timestamp,
                    'token'=>$token,
                    'apikey'=>$api_key,
                    'orderid'=>$service_data->remoteid
                );
                $curl = curl_init();
                curl_setopt($curl, CURLOPT_URL, $url);
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
                $res2 = curl_exec($curl);
                curl_close($curl);
                $res2 = json_decode($res2);

                # Set SSL product data
                $product_type = onlinenic_ssl_getCertName($order_data->productid);
                $server_type = $_REQUEST['server_type'];
                $expiry_date = date_format(date_create($order_data->expire), 'Y-m-d');
                $order_state = $order_data->status;
                $supplier_id = $order_data->orderid;
                $contact_email = $order_data->approvalemail;
                $approval_email = $order_data->approvalemail;
                $contact_set = '';
                $dv_method = isset($order_data->dcvmethod)?$order_data->dcvmethod:'email';
                $csr_data = '';
                $csr = $order_data->csr;
                $cert = $res2->cert;
                $ca_bundle[0] = '';
                $ca_bundle[1] = '';

                # Update DB with SSL data
                $config_data = array(
                    'product_id' => $service_data->remoteid,
                    'supplier_id' => $supplier_id,
                    'product_type' => $product_type,
                    'server_type' => $server_type,
                    'domain' => $domain,
                    'expiry_date' => $expiry_date,
                    'order_state' => $order_state,
                    'contact_email' => $contact_email,
                    'approval_email' => $approval_email,
                    'contact_set' => $contact_set,
                    'dv_method' => $dv_method,
                    'csr_data' => $csr_data,
                    'csr' => $csr,
                    'cert' => $cert,
                    'ca_bundle_1' => $ca_bundle[0],
                    'ca_bundle_2' => $ca_bundle[1]
                );

                # Update database
                DataAccess::UpdateConfigData($id, $config_data);

                # Update next due date
                DataAccess::UpdateNextDueDate($service_data->serviceid, $expiry_date);

                # Set Client side output
                $code .= '<span class="label label-success" style="font-size: 14px;">
                    SSL Certificate</span><textarea style="width: 713px;height: 300px;">'.$cert.'</textarea>';

                break;

            # Approval required state
            case 'pending':
                DataAccess::UpdateStatus($params['serviceid'], 'Configuration Complete');
                $dv_method = isset($order_data->dcvmethod)?$order_data->dcvmethod:'email';
                if($dv_method == 'email'){
                    $msg = 'SSL requires approval from domain owner. Approval email sent to '.$order_data->approvalemail;

                }else if(isset($order_data->filename)){
                    $msg = "Please follow the steps below to complete $dv_method-based Validation
                            <br>Create a folder in your domain name .well-known/pki-validation/ and create file \"{$order_data->filename}\" if it does not already exist.
                            <br> For example:http(s)://$domain/.well-known/pki-validation/{$order_data->filename}
                            <br>Add  follow string to the file \"{$order_data->filename}\" :
                            <br>{$order_data->filecontents}
                            <br>The file contents should be accessible through these URLs publicly on HTTP port 80 or HTTPS port 443.";

                }else if(isset($order_data->dnsrecord)){
                    $msg = "Please follow the steps below to complete $dv_method-based Validation
                            <br>Add the following TXT records below to the DNS server for each domain
                            <br>Add TXT record to common name <strong>$domain</strong> with the value <strong>{$order_data->dnsrecord}</strong>
                            <br>You may have to wait a minute to a couple minutes for the DNS TXT record to propagate 
                            <br>if the TTL record value does not show up or is not the same as the above values.
                            <tr><td>Host</td><td>Type</td><td>Value</td><td>TTL</td></tr>
                            <tr><td>$domain</td><td>TXT</td><td>{$order_data->dnsrecord}</td><td>3600</td></tr>";

                }else if(isset($order_data->md5) && $dv_method == 'dns'){
                    $sha256 = substr($order_data->sha256,0,32).'.'.substr($order_data->sha256,32,63);
                    $uniqueValue = isset($order_data->uniquevalue)?$order_data->uniquevalue:'';
                    $msg = 'DNS based DCV requires the creation of a unique CNAME record, pointed back to Comodo<br>
                                So add following CNAME records to DNS to complete the transfer:<br>
                        <strong>Create CNAME and point </strong><label style=\"color: green\">_'.$order_data->md5.'.EXAMPLE.COM</label><br>
                        to <label style="color: green;">'.$sha256.$uniqueValue.'.comodoca.com</label><br>
                        Note: Please replace EXAMPLE.com with the primary domain in your order.<br> For example, you put web.mail.onlinenic.com in the order, <br>then primary domain name to replace EXAMPLE.com is onlinenic.com.
                        <br>You may have to wait a minute to a couple minutes for the DNS TXT record to propagate <br>if the TTL record value does not show up or is not the same as the above values.';

                }else if(isset($order_data->md5) && $dv_method == 'file'){
                    $upper_md5 = strtoupper($order_data->md5);
                    $msg = "File based DCV requires the creation of a &lt;MD5 hash&gt;.txt and placed in the root of the HTTP server, like so:
                    <br>http://www.example.com/.well-known/pki-validation/<span style=\"color: green\">$upper_md5.txt</span>
                    <br>(Please replace example.com with the Authorization Domain Name. 
                    <br>For example, you put web.mail.abc.com in the order,then replace example.com with abc.com.
                    <br>The file must be created using the UPPERCASE format.) 
                    <br>The file must contain the exact following content in the following order.
                    <br><span style=\"color: green\">{$order_data->sha256}</span>
                    <br><span style=\"color: green\">comodoca.com</span>";
                    if (isset($order_data->uniquevalue)){
                        $cp_uniqueValue = ltrim($order_data->uniquevalue,'.');
                        $msg .= "<br><span style=\"color: green\">$cp_uniqueValue</span>";
                    }
                }else{
                    $msg = '';
                }
                $code .= '<span class="label label-success" style="font-size: 14px;">
                     '. $msg . '</span>';
                break;
            # Approval required state
            case 'in-progress':
                $code .= '<span class="label label-success" style="font-size: 14px;"> Order in-progress </span>';
                break;
        }
    } else {
        # Show configuration option
        $code .= '<td><a class="btn btn-success" href="configuressl.php?cert='.md5($service_id).'">'.onlinenic_ssl_translate('configure_now').'</a></td></tr>';
    }

    # Set closing tag
    $code .= '</table>';

    # Return output
    return $code;
}

/**
 * Renew SSL Order
 * @param array $params::Data from WHMCS
 * @return array::Operation status
 */
function onlinenic_ssl_Renew($params) {
    $orderId =$params['model']['attributes']['orderid'];

    # Get order data
    $order_data = DataAccess::GetSSLProduct($orderId);

    # Get service data
    $service_data = DataAccess::GetServiceID($params['serviceid']);

    # Get config data
    $config_data = json_decode($order_data->configdata);

    # Set CSR
    $csr = $config_data->csr;
    $subject = openssl_csr_get_subject($csr);
    $domain = $subject['CN'];

    # Get billing cycle
    switch ($service_data->billingcycle) {
        case 'Annually':
            $period = 1;
            break;
        case 'Biennially':
            $period = 2;
            break;
        case 'Triennially':
            $period = 3;
            break;
        default:
            $period = 1;
            break;
    }

    # Set order ID
    $up_order_id = $order_data->remoteid;

    # Set approval email
    $approval_email = $config_data->approval_email;

    if($params['configoption5'] == 'on'){
        $username = 135610;
        $password = 654123;
        $api_key = $params['configoption4'];
        $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=renewSSL";
    }else{
        $username = $params['configoption1'];
        $password = $params['configoption7'];
        $api_key = $params['configoption2'];
        $url = "https://onlinenic.com/api4/ssl/index.php?command=renewSSL";
    }
    $timestamp = time();
    $token = md5($username.md5($password).$timestamp."renewSSL");
    $data = array(
        'user'=>$username,
        'timestamp'=>$timestamp,
        'token'=>$token,
        'apikey'=>$api_key,
        'orderid'=>$up_order_id,
        'approvalemail'=>$approval_email,
        'period'=>intval($period) * 12,
        'csr'=>$csr,
    );
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
    $res = curl_exec($curl);
    curl_close($curl);
    $res = json_decode($res);

    # Update DB with new Order ID
    if($res->code == 1000){
        $new_up_order_id = $res->data->orderid;
        if($params['configoption5'] == 'on'){
            $username = 135610;
            $password = 654123;
            $api_key = $params['configoption4'];
            $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }else{
            $username = $params['configoption1'];
            $password = $params['configoption7'];
            $api_key = $params['configoption2'];
            $url = "https://onlinenic.com/api4/ssl/index.php?command=getSSLOrderInfo";
        }
        $timestamp = time();
        $token = md5($username.md5($password).$timestamp."getSSLOrderInfo");
        $data = array(
            'user'=>$username,
            'timestamp'=>$timestamp,
            'token'=>$token,
            'apikey'=>$api_key,
            'orderid'=>$new_up_order_id,
        );
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $res = curl_exec($curl);
        curl_close($curl);
        $res = json_decode($res);
        # Get SSL order details from OnlineNIC
        $new_order_data = $res->data;

        # Set SSL product data
        $product_type = onlinenic_ssl_getCertName($new_order_data->productid);
        $server_type = $_REQUEST['server_type'];
        $expiry_date = '';
        $order_state = $new_order_data->status;
        $supplier_id = $new_order_data->orderid;
        $contact_email = $new_order_data->approvalemail;
        $approval_email = $new_order_data->approvalemail;
        $contact_set = '';
        $dv_method = isset($new_order_data->dcvmethod)?$new_order_data->dcvmethod:'email';
        $csr_data = $subject;
        $cert = '';
        $ca_bundle[0] = '';
        $ca_bundle[1] = '';

        # Update DB with SSL data
        $config_data = array(
            'product_id' => $supplier_id,
            'supplier_id' => $supplier_id,
            'product_type' => $product_type,
            'server_type' => $server_type,
            'domain' => $domain,
            'expiry_date' => $expiry_date,
            'order_state' => $order_state,
            'contact_email' => $contact_email,
            'approval_email' => $approval_email,
            'contact_set' => $contact_set,
            'dv_method' => $dv_method,
            'csr_data' => $csr_data,
            'csr' => $csr,
            'cert' => $cert,
            'ca_bundle_1' => $ca_bundle[0],
            'ca_bundle_2' => $ca_bundle[1]
        );

        # Update database
        DataAccess::UpdateConfigData($orderId, $config_data);

        # Update OpenSRS ID
        DataAccess::UpdateOrder($orderId, 'remoteid', $new_up_order_id);

        # Update Service Data
        DataAccess::UpdateDomain($params['serviceid'], $domain);
        DataAccess::ResetUserInfo('', '', $params['serviceid']);
    } else {
        # Return translated error message
        return onlinenic_ssl_translate($res->msg);
    }
}

/**
 * Display certificate in client area
 * @param array $params::Data from WHMCS
 * @return array $output::Get cert view output
 */
function onlinenic_ssl_getcert($params) {
    # Create smarty instance
    $smartyClass = new Smarty();

    # Set template directory
    $smartyClass->setTemplateDir(dirname(__FILE__).'/src/view/');

    # GC::Check if order exists
    $service_data = DataAccess::GetSSLProduct($params['serviceid']);

    # GC::Check if OpenSRS ID exists
    if($service_data->remoteid) {
        # Create controller instance
        $sslController = new ControllerOnlineNICSSL($params);

        # Get config data
        $config_data = unserialize($service_data->configdata);

        # Set page style
        $style = $smartyClass->fetch('style.AdminProductServicesTab.tpl');

        # Set admin product/services tab variables
        $smartyClass->assign('style', $style);
        $smartyClass->assign('supplier_id', $config_data['supplier_id']);
        $smartyClass->assign('product_type', $config_data['product_type']);
        $smartyClass->assign('contact_email', $config_data['contact_email']);
        $smartyClass->assign('approval_email', $config_data['approval_email']);
        $smartyClass->assign('csr', $config_data['csr']);

        # Set order status
        $order_status = '<span class="label label-success"> '. $config_data['order_state'] .' </span>';

        # Set DV method
        if(!empty($config_data['dv_method'])) {
            $dv_method = '<span class="label label-info"> '. $config_data['dv_method'] .' </span>';
        } else {
            $dv_method = '<span class="label label-warning"> N/A </span>';
        }

        # Set SSL cert
        $cert = $config_data['cert'];

        # Set CA bundle
        $ca_bundle[0] = $config_data['ca_bundle_1'];
        $ca_bundle[1] = $config_data['ca_bundle_2'];

        # Set renewable date
        $renewal_date =  date('Y-m-d', strtotime($config_data['expiry_date']. '-59 days'));
        if(strtotime($renewal_date) < strtotime(date("Y/m/d"))) {
            $renewable_status =  '<span class="label label-success"> Renewable </span>';
        } else {
            $renewable_status =  '<span class="label label-warning"> Not renewable </span>';
        }

        # Set admin product/services tab variables
        $smartyClass->assign('order_status', $order_status);
        $smartyClass->assign('dv_method', $dv_method);
        $smartyClass->assign('renewable', $renewal_date .' | '. $renewable_status);
        $smartyClass->assign('expiry_date', $config_data['expiry_date']);
        $smartyClass->assign('cert', $cert);
        $smartyClass->assign('ca_bundle', $ca_bundle[0] ."\n". $ca_bundle[1]);

        # Get admin product/services tab output
        $ssl_details = $smartyClass->fetch('view.ClientAreaGetCert.tpl');

        # Return output
        return array (
            'templatefile'  =>  'getcert',
            'breadcrumb'    =>  '> <a href="#">Get cert</a>',
            'vars'          =>  array('output' =>  $ssl_details)
        );
    }
}


/**
 * SSL Config Step One
 * @param array $params::Data from WHMCS
 * @return array $fields::Output fields
 */
function onlinenic_ssl_SSLStepOne($params) {

    # Create SSL controller instance
    $sslController = new ControllerOnlineNICSSL($params);

    $_LANG = onlinenic_ssl_loadLanguage();

    $product_type = onlinenic_ssl_getCertType($params['certtype']);
    $types = onlinenic_ssl_getRequiredContacts($product_type);
    $fields = array();

    $previous_type = null;
    $contact_types = array('admin', 'billing', 'tech', 'organization', 'signer');
    foreach($contact_types as $t)
    {
        if(in_array($t, $types))
        {
            $field = ucfirst($t);
            $tr = onlinenic_ssl_translate($t);

            if(in_array($previous_type, array('admin' ,'billing', 'tech', 'organization')))
            {
                $fields['additionalfields'][$tr]['CopyDataFrom'.$previous_type] = array
                (
                    'FriendlyName'  =>  onlinenic_ssl_translate('use_'.$previous_type.'_contact_info'),
                    'Type'          =>  'yesno',
                    'Description'   =>  '<script type="text/javascript">$(function(){$("input[type=checkbox][name=\'fields[CopyDataFrom'.$previous_type.']\']").click(function(){
                            if($(this).is(":checked"))
                            {
                                /* INPUT TYPE TEXT */
                                $("input[type=text][name^=\'fields['.ucfirst($previous_type).'\']").each(function(){
                                    val         = $(this).val();
                                    name        = $(this).attr("name");
                                    new_name    = name.replace("'.ucFirst($previous_type).'", "'.$field.'");
                                    $("input[type=text][name=\'"+new_name+"\']").val(val);
                                    $("input[type=text][name=\'"+new_name+"\']").attr("readonly", "readonly");
                                });

                                $("input[type=text][name^=\'fields['.ucfirst($previous_type).'\']").change(function(){
                                    val         = $(this).val();
                                    name        = $(this).attr("name");
                                    new_name    = name.replace("'.ucFirst($previous_type).'", "'.$field.'");
                                    $("input[type=text][name=\'"+new_name+"\']").val(val);
                                    $("input[type=text][name=\'"+new_name+"\']").change();
                                });

                                /* SELECT */
                                $("select[name^=\'fields['.ucfirst($previous_type).'\']").each(function(){
                                    val         = $(this).val();
                                    name        = $(this).attr("name");
                                    new_name    = name.replace("'.ucFirst($previous_type).'", "'.$field.'");
                                    $("select[name=\'"+new_name+"\']").val(val);
                                    $("select[name=\'"+new_name+"\']").attr("readonly", "readonly");
                                });

                                $("select[name^=\'fields['.ucfirst($previous_type).'\']").change(function(){
                                    val         = $(this).val();
                                    name        = $(this).attr("name");
                                    new_name    = name.replace("'.ucFirst($previous_type).'", "'.$field.'");
                                    $("select[name=\'"+new_name+"\']").val(val);
                                    $("select[name=\'"+new_name+"\']").change();
                                });
                            }
                            else
                            {
                                /* INPUT TYPE TEXT */
                                $("input[type=text][name^=\'fields['.ucfirst($previous_type).'\']").unbind("change");
                                $("input[type=text][name^=\'fields['.$field.'\']").removeAttr("readonly");

                                /* SELECT  */
                                $("select[name^=\'fields['.ucfirst($previous_type).'\']").unbind("change");
                                $("select[name^=\'fields['.$field.'\']").removeAttr("readonly");
                            }
                        })})</script>',
                );
            }

            $fields['additionalfields'][$tr][$field.'FirstName'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('first_name'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  $t != 'organization' ? true : false,
            );
            $fields['additionalfields'][$tr][$field.'LastName'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('last_name'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  $t != 'organization' ? true : false,
            );
            $fields['additionalfields'][$tr][$field.'Title'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('title'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  in_array($t, array('sss123', 'sslwebserver', 'sslwebserv_wildcard', 'sslwebserver_ev', 'securesite', 'securesite_ev', 'securesite_pro', 'securite_site_pro_ev', 'truebizd_ev')) ? true : false,
            );
            $fields['additionalfields'][$tr][$field.'Name'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('organization'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  false,
            );
            $fields['additionalfields'][$tr][$field.'Address1'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('address1'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  true,
            );
            $fields['additionalfields'][$tr][$field.'Address2'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('address2'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  false,
            );
            $fields['additionalfields'][$tr][$field.'Address3'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('address3'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  false,
            );
            $fields['additionalfields'][$tr][$field.'City'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('city'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  true,
            );
            $fields['additionalfields'][$tr][$field.'State'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('state'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  true,
            );
            $fields['additionalfields'][$tr][$field.'PostalCode'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('postal_code'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  false,
            );
            $fields['additionalfields'][$tr][$field.'Country'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('country'),
                'Type'          =>  'country',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  true,
            );
            if(in_array('san', $types)){
                $fields['additionalfields'][$tr][$field.'Phone'] = array
                (
                    'FriendlyName'  =>  onlinenic_ssl_translate('Phone Number'),
                    'Type'          =>  'text',
                    'Size'          =>  '30',
                    'Description'   =>  onlinenic_ssl_translate('phone_description'),
                    'Required'      =>  true,
                );
            }else{
                $fields['additionalfields'][$tr][$field.'Phone'] = array
                (
                    'FriendlyName'  =>  onlinenic_ssl_translate('phone'),
                    'Type'          =>  'text',
                    'Size'          =>  '30',
                    'Description'   =>  onlinenic_ssl_translate('phone_description'),
                    'Required'      =>  true,
                );
            }

            $fields['additionalfields'][$tr][$field.'Fax'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('fax'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  false,
            );
            $fields['additionalfields'][$tr][$field.'Email'] = array
            (
                'FriendlyName'  =>  onlinenic_ssl_translate('Email Address'),
                'Type'          =>  'text',
                'Size'          =>  '30',
                'Description'   =>  '',
                'Required'      =>  true,
            );
            $previous_type = $t;
        }
    }

    # Module log
    $sslController->ModuleLog(__FUNCTION__, $fields, '', '', $params);
    return $fields;
}

function onlinenic_ssl_SSLStepTwo(&$params) {
    //LOAD LANG
    $_LANG = onlinenic_ssl_loadLanguage();

    //CHECK TYPES AND PHONES
    $product_type = onlinenic_ssl_getCertType($params['certtype']);
    $configData = $params['sslOrder']['attributes']['configdata'];
    $configData = json_decode($configData);
    $types = array('Admin','Billing','Tech','Organization','Signer');
    foreach($types as $type)
    {
        $n = $type.'Phone';
        if(isset($configData->fields->$n))
        {
            $phone = $configData->fields->$n;
            $preg = preg_match('/(\+{1})([0-9]{2,3})(\.{1})([0-9]+)/', $phone);
            if(!$preg)
            {
                return array('error'    =>  onlinenic_ssl_translate('invalid_phone_number'));
            }
        }
    }

    $dcvMethod = $_POST['dcvMethod'];
    $san = $_REQUEST['san'];
    $subject  = openssl_csr_get_subject($configData->csr);
    $domain = $subject['CN'];
    if($params['configoption5'] == 'on'){
        $username = 135610;
        $password = 654123;
        $api_key = $params['configoption4'];
        $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=getApprovalEmailList";
    }else{
        $username = $params['configoption1'];
        $password = $params['configoption7'];
        $api_key = $params['configoption2'];
        $url = "https://onlinenic.com/api4/ssl/index.php?command=getApprovalEmailList";
    }
    $emails = array();
    $timestamp = time();
    $token = md5($username.md5($password).$timestamp."getApprovalEmailList");
    $data = array(
        'user'=>$username,
        'timestamp'=>$timestamp,
        'token'=>$token,
        'apikey'=>$api_key,
        'domain'=>$domain,
    );
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
    $res = curl_exec($curl);
    curl_close($curl);
    $res = json_decode($res);
    if($res->code != '1000'){
        return array('error'    =>  onlinenic_ssl_translate($res->msg));
    }
    $emails = $res->data;
    //Log
    logModuleCall(__FUNCTION__, "Get approval email", $params, $res);
    return array
    (
        'approveremails'   =>   array($dcvMethod,$emails,$san),
    );
}

function onlinenic_ssl_SSLStepThree(&$params)
{
    $_LANG = onlinenic_ssl_loadLanguage();

    $orderId = $params['model']['attributes']['orderid'];
    $configData = $params['sslOrder']['attributes']['configdata'];
    $configData = json_decode($configData);
    $subject  = openssl_csr_get_subject($configData->csr);
    $domain = $subject['CN'];
    $dcvMethod = $_POST['dcvmethod'];
    $san = $_POST['san'];
    # Get service data
    $service_data = DataAccess::GetServiceID($params['serviceid']);

    # Get billing cycle
    switch ($service_data->billingcycle) {
        case 'Annually':
            $period = 1;
            break;
        case 'Biennially':
            $period = 2;
            break;
        case 'Triennially':
            $period = 3;
            break;
        default:
            $period = 1;
            break;
    }

    //Cert Type
    $product_type = onlinenic_ssl_getCertType($params['configoption3']);
    //Order from OnlineNIC
    if($params['configoption5'] == 'on'){
        $username = 135610;
        $password = 654123;
        $api_key = $params['configoption4'];
        $url = "http://ote.onlinenic.com/api4/ssl/index.php?command=orderSSL";
    }else{
        $username = $params['configoption1'];
        $password = $params['configoption7'];
        $api_key = $params['configoption2'];
        $url = "http://api.onlinenic.com/api4/ssl/index.php?command=orderSSL";
    }
    $timestamp = time();
    $token = md5($username.md5($password).$timestamp."orderSSL");
    $data = array(
        'user'=>$username,
        'timestamp'=>$timestamp,
        'token'=>$token,
        'apikey'=>$api_key,
        'productid'=>$product_type,
        'period'=>intval($period) * 12,
        'dcvmethod'=>$dcvMethod,
        'approvalemail'=>$configData->approveremail,
        'servertype'=>2,
        'hashingalgorithm'=>'',
        'csr'=>$configData->csr,
        'sans'=>$san,
        'adminfirstname'=>$configData->fields->AdminFirstName,
        'adminlastname'=>$configData->fields->AdminLastName,
        'admintitle'=>$configData->fields->AdminTitle,
        'adminphone'=>$configData->fields->AdminPhone,
        'adminemail'=>$configData->fields->AdminEmail,
        'techfirstname'=>$configData->fields->TechFirstName,
        'techlastname'=>$configData->fields->TechLastName,
        'techtitle'=>$configData->fields->TechTitle,
        'techphone'=>$configData->fields->TechPhone,
        'techemail'=>$configData->fields->TechEmail,
        'orgname'=>$configData->fields->OrganizationName,
        'orgaddressline1'=>$configData->fields->OrganizationAddress1,
        'orgaddressline2'=>$configData->fields->OrganizationAddress2,
        'orgstate'=>$configData->fields->OrganizationState,
        'orgcity'=>$configData->fields->OrganizationCity,
        'orgcountry'=>$configData->fields->OrganizationCountry,
        'orgphone'=>$configData->fields->OrganizationPhone,
        'orgfax'=>$configData->fields->OrganizationFax,
        'orgpostalcode'=>$configData->fields->OrganizationPostalCod,
    );
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
    $res = curl_exec($curl);
    curl_close($curl);
    $res = json_decode($res);
    if($res->code == '1000') {
        # Get order ID
        $up_order_id = $res->data->orderid;

        # Set SSL product data
        $server_type = $params['configdata']['servertype'];
        $expiry_date = '';
        $order_state = 'PENDING';
        $dv_method = 'email';
        $cert = '';
        $ca_bundle[0] = '';
        $ca_bundle[1] = '';

        # Update DB with SSL data
        $config_data = array(
            'product_id' => $up_order_id,
            'supplier_id' => $up_order_id,
            'product_type' => $product_type,
            'server_type' => $server_type,
            'domain' => $domain,
            'expiry_date' => $expiry_date,
            'order_state' => $order_state,
            'contact_email' => $configData->fields->AdminEmail,
            'approval_email' => $configData->approveremail,
            'contact_set' => '',
            'dv_method' => $dv_method,
            'csr_data' => $subject,
            'csr' => $configData->csr,
            'cert' => $cert,
            'ca_bundle_1' => $ca_bundle[0],
            'ca_bundle_2' => $ca_bundle[1]
        );
        if(isset($res->data->file_name)){
            $config_data['file_name'] = $res->data->file_name;
        }
        if(isset($res->data->file_record)){
            $config_data['file_record'] = $res->data->file_record;
        }
        if(isset($res->data->dns_record)){
            $config_data['dns_record'] = $res->data->dns_record;
        }
        if(isset($res->data->md5)){
            $config_data['md5'] = $res->data->md5;
        }
        if(isset($res->data->sha256)){
            $config_data['sha256'] = $res->data->sha256;
        }
        $config_data = array_merge($params['configdata'],$config_data);
        array_merge($params,$config_data);

        # Update database
        DataAccess::UpdateConfigData($orderId, $config_data);
        $params['configdata'] = $config_data;
        $params['sslOrder']['original']['configdata'] = $config_data;
        $params['sslOrder']['attributes']['configdata'] = $config_data;

        # Update OpenSRS ID
        DataAccess::UpdateOrder($orderId, 'remoteid', $up_order_id);

        # Set next due date
        $next_due_date = '';
        switch ($period) {
            case '1':
                $next_due_date = date('Y-m-d', strtotime($expiry_date. '+365 days'));;
                break;
            case '2':
                $next_due_date = date('Y-m-d', strtotime($expiry_date. '+730 days'));;
                break;
            case '3':
                $next_due_date = date('Y-m-d', strtotime($expiry_date. '+1095 days'));;
                break;
        }

        # Update Service Data
        DataAccess::UpdateDomain($params['serviceid'], $domain);
        DataAccess::UpdateStatus($params['serviceid'], 'Configuration Complete');
        DataAccess::UpdateNextDueDate($params['serviceid'], $next_due_date);
        DataAccess::ResetUserInfo('', '', $params['serviceid']);
    } else {
        return array('error' => onlinenic_ssl_translate($res->msg));
    }
}


function onlinenic_ssl_terminateaccount($params)
{
    //get cert details
    $q = mysql_safequery_Oln("SELECT * FROM tblsslorders WHERE serviceid = ?", array($params['serviceid']));
    $row = mysql_fetch_assoc($q);
    if($row['remoteid'])
    {
        $openSRS = new OpenSRS($params['configoption1'], 0, $params['configoption2'], $params['configoption3'] == 'on' ? 0 : 1);
        $send = array
        (
            'action'        =>  'cancel_order',
            'object'        =>  'trust_service',
            'attributes'    =>  array
            (
                'order_id'  =>  $row['remoteid']
            )
        );

        $res = $openSRS->send($send);
        if(!$openSRS->isSuccess())
        {
            return onlinenic_ssl_translate($openSRS->getError());
        }
    }


    $q = mysql_safequery_Oln("DELETE FROM tblsslorders WHERE serviceid = ?", array($params['serviceid']));
    return 'success';
}

function onlinenic_ssl_admincustombuttonarray()
{
    return array
    (
        onlinenic_ssl_translate('resend_cert_email')      =>  'resend',
        onlinenic_ssl_translate('resend_approve_email')   =>  'resendApproveEmail',
        onlinenic_ssl_translate('renew')                  =>  'Renew',
    );
}

function onlinenic_ssl_ClientAreaCustomButtonArray()
{
    return array
    (
        onlinenic_ssl_translate('get_certificate')   =>  'getcert'
    );
}


/**
 * Resend cert email to client
 * @param type $params
 * @return type
 */
function onlinenic_ssl_resend($params)
{
    $q = mysql_safequery_Oln("SELECT * FROM tblsslorders WHERE serviceid = ?", array($params['serviceid']));
    $row = mysql_fetch_assoc($q);

    $openSRS = new OpenSRS($params['configoption1'], 0, $params['configoption2'], $params['configoption3'] == 'on' ? 0 : 1);
    $send = array
    (
        'action'        =>  'resend_cert_email',
        'object'        =>  'trust_service',
        'attributes'    =>  array
        (
            'order_id'  =>  $row['remoteid']
        )
    );

    $res = $openSRS->send($send);
    if(!$openSRS->isSuccess())
    {
        return onlinenic_ssl_translate($openSRS->getError());
    }

    return 'success';
}

function onlinenic_ssl_resendApproveEmail($params)
{
    //ORDER
    $q = mysql_safequery_Oln("SELECT * FROM tblsslorders WHERE serviceid = ?", array($params['serviceid']));
    $cert = mysql_fetch_assoc($q);
    $order_id = $cert['remoteid'];

    $openSRS = new OpenSRS($params['configoption1'], 0, $params['configoption2'], $params['configoption3'] == 'on' ? 0 : 1);
    $send = array
    (
        'action'        =>  'resend_approve_email',
        'object'        =>  'trust_service',
        'attributes'    =>  array
        (
            'order_id'  =>  $order_id,
        )
    );

    $r = $openSRS->send($send);
    if(!$openSRS->isSuccess())
    {
        return $openSRS->getError();
    }

    return 'success';
}

/**************************** HELPER FUNCTIONS ************************************/

/**
 * Load Language File
 * @return array
 */
function LoadLang_Oln() {
    # Set global variable
    GLOBAL $onlinenic_ssl_language;

    if($onlinenic_ssl_language)
        return $onlinenic_ssl_language;

    # Initialize variable
    $language = DataAccess::GetLanguage();

    # Load user's language file
    $langfilename = dirname(__FILE__).DS.'lang'.DS.$language.'.php';

    # Set default language file
    $deflangfilename = dirname(__FILE__).DS.'lang'.DS.'english.php';

    # Try loading user's language file
    if(file_exists($langfilename)) {
        include($langfilename);
    } else {
        include($deflangfilename);
    }

    # Set language file
    $onlinenic_ssl_language = $_LANG;

    # Return data
    return $_LANG;
}

/**
 * Translate Text
 * @param int $key::Key name
 * @return string::Translated text
 */
function Translate_Oln($key) {
    # Load language file
    $_LANG = LoadLang_Oln();

    if(isset($_LANG[$key])) {
        # Return translation
        return $_LANG[$key];
    } else {
        # Return default
        return $key;
    }
}



/**
 * SSL Server Types
 * @return string::Server Type
 */
function onlinenic_ssl_whmcsServerTypes() {
    return array(
        # WHMCS Type
        "1001"  =>  "AOL",
        "1002"  =>  "Apache +ModSSL",
        "1003"  =>  "Apache-SSL",
        "1004"  =>  "C2Net Stronghold",
        "1005"  =>  "Cobalt Raq",
        "1006"  =>  "Covalent Server Software",
        "1031"  =>  "cPanel / WHM",
        "1029"  =>  "Ensim",
        "1032"  =>  "H-Sphere",
        "1007"  =>  "IBM HTTP Server",
        "1008"  =>  "IBM Internet Connection Server",
        "1009"  =>  "iPlanet",
        "1010"  =>  "Java Web Server",
        "1011"  =>  "Lotus Domino",
        "1012"  =>  "Lotus Domino Go!",
        "1013"  =>  "Microsoft IIS 1.x to 4.x",
        "1014"  =>  "Microsoft IIS 5.x and later",
        "1015"  =>  "Netscape Enterprise Server",
        "1016"  =>  "Netscape FastTrack",
        "1017"  =>  "Novell Web Server",
        "1018"  =>  "Oracle",
        "1030"  =>  "Plesk",
        "1019"  =>  "Quid Pro Quo",
        "1020"  =>  "R3 SSL Server",
        "1021"  =>  "Raven SSL",
        "1022"  =>  "RedHat Linux",
        "1023"  =>  "SAP Web Application Server",
        "1024"  =>  "Tomcat",
        "1025"  =>  "Website Professional",
        "1026"  =>  "WebStar 4.x and later",
        "1027"  =>  "WebTen",
        "1028"  =>  "Zeus Web Server",
        "2001"  =>  "Apache 2",
        "2002"  =>  "Apache + OpenSSL",
        "2003"  =>  "C2NET",
        "2004"  =>  "Cobalt Series",
        "2005"  =>  "H-Sphere",
        "2006"  =>  "Citrix",
        "2007"  =>  'IPSwitch',
        "2008"  =>  'Web Logic',
        "1000"  =>  "Other (not listed)"
    );
}

//A LITTLE HELPER
if(function_exists('mysql_safequery_Oln') == false) {
    function mysql_safequery_Oln($query,$params=false) {
        if ($params) {
            foreach ($params as &$v) { $v = mysql_real_escape_string($v); }
            $sql_query = vsprintf( str_replace("?","'%s'",$query), $params );
            $sql_query = mysql_query($sql_query);
        } else {
            $sql_query = mysql_query($query);
        }
        return ($sql_query);
    }
}

function onlinenic_ssl_getServerType($server, $type)
{
    $comodo = array
    (
        "1001"  =>  "other",           // AOL
        "1002"  =>  "apachessl",          // Apache +ModSSL
        "1003"  =>  "other",    // Apache-SSL (Ben-SSL, not Stronghold)
        "1004"  =>  "other",              // C2Net Stronghold
        "1005"  =>  "other",       // Cobalt Raq
        "1006"  =>  "other",              // Covalent Server Software
        "1031"  =>  "whmcpanel",             // cPanel / WHM
        "1029"  =>  "ensim",              // Ensim
        "2005"  =>  "hsphere",            // H-Sphere
        "1007"  =>  "ibmhttp",            // IBM HTTP Server
        "1008"  =>  "ibmhttp",            // IBM Internet Connection Server
        "1009"  =>  "other",            // iPlanet
        "1010"  =>  "javawebserver",              // Java Web Server (Javasoft / Sun)
        "1011"  =>  "domino",             // Lotus Domino
        "1012"  =>  "other",       // Lotus Domino Go!
        "1013"  =>  "iis4",               // Microsoft IIS 1.x to 4.x
        "1014"  =>  "iis5",               // Microsoft IIS 5.x and later
        "1015"  =>  "netscape",           // Netscape Enterprise Server
        "1016"  =>  "netscape",           // Netscape FastTrack
        "1017"  =>  "novell",              // Novell Web Server
        "1018"  =>  "oracle",              // Oracle
        "1000"  =>  "other",              // Other (not listed)
        "1030"  =>  "plesk",              // Plesk
        "1019"  =>  "other",              // Quid Pro Quo
        "1020"  =>  "other",              // R3 SSL Server
        "1021"  =>  "other",              // Raven SSL
        "1022"  =>  "redhat",              // RedHat Linux
        "1023"  =>  "sap",              // SAP Web Application Server
        "1024"  =>  "tomcat",             // Tomcat
        "1025"  =>  "other",            // Website Professional
        "1026"  =>  "webstar",            // WebStar 4.x and later
        "1027"  =>  "other",              // WebTen (from Tenon)
        "1028"  =>  "other",             // Zeus Web Server
        //CUSTOM
        "2001"  =>  "other",                //Apache 2
        "2002"  =>  "other",                //Apache + OpenSSL
        "2003"  =>  "other",                //C2NET
        "2004"  =>  "other",                //Cobalt Series
        "2005"  =>  "other",                //H-Sphere
        "2006"  =>  "citrix",               //Citrix
        "2007"  =>  "other",                //IP Switch
        "2008"  =>  "webstart"              //WebLogic
    );

    $other = array
    (
        "1001"  =>  'other',              // AOL
        "1002"  =>  "apacheapachessl",          // Apache +ModSSL
        "1003"  =>  "other",    // Apache-SSL (Ben-SSL, not Stronghold)
        "1004"  =>  "c2net",              // C2Net Stronghold
        "1005"  =>  "cobaltseries",       // Cobalt Raq
        "1006"  =>  'other',              // Covalent Server Software
        "1031"  =>  "cpanel",             // cPanel / WHM
        "1029"  =>  "ensim",              // Ensim
        "2005"  =>  "hsphere",            // H-Sphere
        "1007"  =>  "ibmhttp",            // IBM HTTP Server
        "1008"  =>  "ibmhttp",            // IBM Internet Connection Server
        "1009"  =>  "iplanet",            // iPlanet
        "1010"  =>  'other',              // Java Web Server (Javasoft / Sun)
        "1011"  =>  "domino",             // Lotus Domino
        "1012"  =>  "dominogo4626",       // Lotus Domino Go!
        "1013"  =>  "iis4",               // Microsoft IIS 1.x to 4.x
        "1014"  =>  "iis5",               // Microsoft IIS 5.x and later
        "1015"  =>  "netscape",           // Netscape Enterprise Server
        "1016"  =>  "netscape",           // Netscape FastTrack
        "1017"  =>  'other',              // Novell Web Server
        "1018"  =>  "other",              // Oracle
        "1000"  =>  "other",              // Other (not listed)
        "1030"  =>  "plesk",              // Plesk
        "1019"  =>  "other",              // Quid Pro Quo
        "1020"  =>  "other",              // R3 SSL Server
        "1021"  =>  "other",              // Raven SSL
        "1022"  =>  "other",              // RedHat Linux
        "1023"  =>  "other",              // SAP Web Application Server
        "1024"  =>  "tomcat",             // Tomcat
        "1025"  =>  "website",            // Website Professional
        "1026"  =>  "webstar",            // WebStar 4.x and later
        "1027"  =>  "other",              // WebTen (from Tenon)
        "1028"  =>  "zeusv3",             // Zeus Web Server
        //CUSTOM
        "2001"  =>  "apache2",              //Apache 2
        "2002"  =>  "apacheopenssl",        //Apache + OpenSSL
        "2003"  =>  "c2net",                //C2NET
        "2004"  =>  "cobaltseries",         //Cobalt Series
        "2005"  =>  "hsphere",              //H-Sphere
        "2006"  =>  "other",                //Citrix
        "2007"  =>  "ipswitch",             //IP Switch
        "2008"  =>  "weblogic"              //WebLogic
    );

    $comodo_types = array('comodo_ev','comodo_instantssl','essentialssl', 'essentialssl_wildcard', 'comodo_premiumssl', 'comodo_premiumssl_wildcard','comodo_ssl','comodo_wildcard','positivessl');
    if(in_array($type, $comodo_types))
        return $comodo[$server];

    return $other[$server];
}

function onlinenic_ssl_getRequiredContacts($t)
{
    $contacts = array
    (
        'admin'         =>  array
        (
            //GEOTRUST
            'ssl11',
            'ssl12',
            'ssl13',
            'ssl14',
            'ssl15',
            'ssl16',
            'ssl17',
            //SYMANTEC
            'ssl21',
            'ssl22',
            'ssl23',
            'ssl24',
            'ssl25',
            //COMODO
            'ssl31',
            'ssl32',
            'ssl33',
            'ssl34',
            'ssl35',
            'ssl41',
            'ssl42',
            'ssl43',
            'ssl44',
            'ssl45',
            'ssl46',
            'ssl47',
        ),
        'tech'          =>  array
        (
            //GEOTRUST
            'ssl11',
            'ssl12',
            'ssl13',
            'ssl14',
            'ssl15',
            'ssl16',
            'ssl17',
            //SYMANTEC
            'ssl21',
            'ssl22',
            'ssl23',
            'ssl24',
            'ssl25',
            //COMODO
            'ssl31',
            'ssl33',
            'ssl34',
            'ssl35',
            'ssl41',
            'ssl42',
            'ssl43',
            'ssl44',
            'ssl45',
            'ssl46',
            'ssl47',
        ),
        'organization'  =>  array
        (
            //GEOTRUST
            'ssl14',
            'ssl15',
            'ssl16',
            //SYMANTEC
            'ssl21',
            'ssl22',
            'ssl23',
            'ssl24',
            'ssl25',
            //COMODO
            'ssl34',
            'ssl35',
            'ssl43',
            'ssl44',
            'ssl45',
            'ssl47',
        ),
        'signer'        =>  array
        (
            //GEOTRUST
            'ssl11',
            'ssl12',
            'ssl13',
            'ssl17',
            //COMODO
            'ssl31',
            'ssl33',
            'ssl41',
            'ssl42',
            'ssl46',
        ),
//        'san' =>  array
//        (
//            'ssl31',//test
//            'ssl14',
//            'ssl15',
//            'ssl33',
//            'ssl35',
//            'ssl46',
//            'ssl45',
//            'ssl47',
//            'ssl21',
//            'ssl22',
//            'ssl23',
//            'ssl24',
//            'ssl25',
//        ),
    );

    $types = array();
    foreach($contacts as $type => $values)
    {
        if(in_array($t, $values))
            $types[] = $type;
    }

    return $types;
}

function onlinenic_ssl_getCertType($type = null)
{
    $certs = array
    (
        //GEO TRUST
        'GeoTrust - Rapid SSL'                               =>  'ssl11',
        'GeoTrust - RapidSSL Wildcard'                       =>  'ssl12',
        'GeoTrust - QuickSSL Premium'                        =>  'ssl13',
        'GeoTrust - True BusinessID'                         =>  'ssl14',
        'GeoTrust - True BusinessID with EV'                 =>  'ssl15',
        'GeoTrust - True BusinessID Wildcard'                =>  'ssl16',
        'GeoTrust - QuickSSL Premium Wildcard'               =>  'ssl17',
        //SYMANTEC
        'Symantec - Secure Site'                             =>  'ssl21',
        'Symantec - Secure Site Pro'                         =>  'ssl22',
        'Symantec - Secure Site with EV'                     =>  'ssl23',
        'Symantec - Secure Site Pro with EV'                 =>  'ssl24',
        'Symantec - Secure Site Wildcard'                    =>  'ssl25',

        //COMODO
        'Comodo - PositiveSSL'                               =>  'ssl31',
        'Comodo - PositiveSSL Wildcard'                      =>  'ssl32',
//        'Comodo - PositiveSSL Multi-Domain'                  =>  'ssl33',
        'Comodo - Positive EV SSL'                           =>  'ssl34',
//        'Comodo - Positive EV Multi-Domain SSL'              =>  'ssl35',
        'Comodo - COMODO SSL'                                =>  'ssl41',
        'Comodo - COMODO SSL Wildcard'                       =>  'ssl42',
        'Comodo - COMODO PremiumSSL Wildcard'                =>  'ssl43',
        'Comodo - COMODO EV SSL'                             =>  'ssl44',
//        'Comodo - COMODO EV Multi-Domain SSL'                =>  'ssl45',
//        'Comodo - COMODO SSL UCC'                            =>  'ssl46',
//        'Comodo - COMODO Multi-Domain SSL'                   =>  'ssl47',
    );

    if(isset($type))
        return $certs[$type];

    return $certs;
}

function onlinenic_ssl_getCertName($type = null)
{
    $certs = array
    (
        //GEO TRUST
        'ssl11'                             =>  'GeoTrust - Rapid SSL' ,
        'ssl12'                       =>  'GeoTrust - RapidSSL Wildcard',
        'ssl13'                        =>  'GeoTrust - QuickSSL Premium',
        'ssl14'                         =>  'GeoTrust - True BusinessID',
        'ssl15'                =>  'GeoTrust - True BusinessID with EV' ,
        'ssl16'         => 'GeoTrust - True BusinessID Wildcard'       ,
        'ssl17'               =>  'GeoTrust - QuickSSL Premium Wildcard',
        //SYMANTEC
        'ssl21'                   => 'Symantec - Secure Site'          ,
        'ssl22'                        =>  'Symantec - Secure Site Pro' ,
        'ssl23'                  => 'Symantec - Secure Site with EV'   ,
        'ssl24'            =>  'Symantec - Secure Site Pro with EV'     ,
        'ssl25'                => 'Symantec - Secure Site Wildcard'     ,

        //COMODO
        'ssl31'                            =>  'Comodo - PositiveSSL'   ,
        'ssl32'                    =>  'Comodo - PositiveSSL Wildcard'  ,
        'ssl33'              =>  'Comodo - PositiveSSL Multi-Domain'    ,
        'ssl34'                         =>'Comodo - Positive EV SSL'   ,
        'ssl35'             => 'Comodo - Positive EV Multi-Domain SSL'  ,
        'ssl41'                          =>'Comodo - COMODO SSL'       ,
        'ssl42'                 => 'Comodo - COMODO SSL Wildcard'       ,
        'ssl43'                =>  'Comodo - COMODO PremiumSSL Wildcard',
        'ssl44'                      =>  'Comodo - COMODO EV SSL'       ,
        'ssl45'             =>   'Comodo - COMODO EV Multi-Domain SSL'   ,
        'ssl46'                           => 'Comodo - COMODO SSL UCC' ,
        'ssl47'                  => 'Comodo - COMODO Multi-Domain SSL' ,
    );

    if(isset($type))
        return $certs[$type];

    return $certs;
}



// Old method (Deprecated)
function onlinenic_ssl_loadLanguage() {
    # Set global variable
    GLOBAL $onlinenic_ssl_language;

    if($onlinenic_ssl_language)
        return $onlinenic_ssl_language;

    # Initialize variable
    $language = DataAccess::GetLanguage();

    # Load user's language file
    $langfilename = dirname(__FILE__).DS.'lang'.DS.$language.'.php';

    # Set default language file
    $deflangfilename = dirname(__FILE__).DS.'lang'.DS.'english.php';

    # Try loading user's language file
    if(file_exists($langfilename)) {
        include($langfilename);
    } else {
        include($deflangfilename);
    }

    # Set language file
    $onlinenic_ssl_language = $_LANG;

    # Return data
    return $_LANG;
}
// Old method (Deprecated)
function onlinenic_ssl_translate($key) {
    # Load language file
    $_LANG = onlinenic_ssl_loadLanguage();

    if(isset($_LANG[$key])) {
        # Return translation
        return $_LANG[$key];
    } else {
        # Return default
        return $key;
    }
}
