<?php
# Define constants
define('DS', DIRECTORY_SEPARATOR);
define('PS', PATH_SEPARATOR);
define('CRLF', "\r\n");

# Set required files
require_once(dirname(dirname(__FILE__)).DS.'vendor'.DS.'autoload_oln.php');

# Set Laravel class alias
use Illuminate\Database\Capsule\Manager as Capsule;

# Set alias to Database access class
use DataAccess\DataAccess as DataAccess;

# Use OpenSRS request class
use opensrs\Request;

/**
 * OpenSRS Controller Class
 */
class ControllerOnlineNICSSL {
    # Class variables
    protected static $smartyClass = '';

    /**
     * ControllerOpensrsSSL constructor.
     * @param array $params::Data from WHMCS
     */
    public function __construct($params) {
        # Define constants
        define('OSRS_USERNAME', $params['configoption1']);

        # Handle test mode
        if($params['configoption5'] == 'on') {
            # Set test system
            define('OSRS_KEY', $params['configoption4']);
            define('OSRS_HOST', 'horizon.opensrs.net');
        } else {
            # Set live system
            define('OSRS_KEY', $params['configoption2']);
            define('OSRS_HOST', 'rr-n1-tor.opensrs.net');
        }

        # Create smarty instance
        self::$smartyClass = new Smarty();

        # Set template directory
        self::$smartyClass->setTemplateDir(dirname(dirname(__FILE__)).'/view/');
    }

    /**
     * Get Certificate
     * @param int $product_id::SSL product ID
     * @return array $response::Response from OpenSRS
     */
    public function GetCert($product_id) {
        # Create new OpenSRS lib class
        $opensrsLib = new Request();

        # Set request data
        $request_data = array(
            'func' => 'trustGetCert',
            'data' => array(
                'attributes' => array(
                    'product_id' => $product_id,
                    'return_ca_certs' => 1
                )
            )
        );

        try {
            # Execute get SSL details request
            $response = $opensrsLib->process('array', $request_data);
        } catch (\opensrs\APIException $e) {
            # Return exception
            $error_msg = array(
                'is_success' => False,
                'error' => $e->getMessage()
            );
            return $error_msg;
        }

        # Module log
        self::ModuleLog(__FUNCTION__, $request_data, $response->resultFullRaw, '', '');

        if($response->resultFullRaw['is_success'] == True) {
            return $response->resultFullRaw;
        } else {
            return False;
        }
    }

    /**
     * Get Product Info
     * @param array $order_id::Order ID
     * @return mixed::Product info from OpenSRS or operation status
     */
    public function GetProduct($order_id) {
        # Create new OpenSRS lib class
        $opensrsLib = new Request();

        # Set request data
        $request_data = array(
            'func' => 'trustGetProductInfo',
            'data' => array(
                'attributes' => array(
                    'product_id' => $order_id,
                    'all_info' => 1
                )
            )
        );

        try {
            # Execute get SSL details request
            $response = $opensrsLib->process('array', $request_data);
        } catch (\opensrs\APIException $e) {
            # Return exception
            $error_msg = array(
                'is_success' => False,
                'error' => $e->getMessage()
            );
            return $error_msg;
        }

        # Module log
        self::ModuleLog(__FUNCTION__, $request_data, $response->resultFullRaw, '', '');

        if($response->resultFullRaw['is_success'] == True) {
            return $response->resultFullRaw;
        } else {
            return False;
        }
    }

    /**
     * Get Order Info
     * @param int $order_id::Order ID
     * @return mixed=::Order ID info from OpenSRS or Operation status
     */
    public function GetOrderInfo($order_id) {
        # Create new OpenSRS lib class
        $opensrsLib = new Request();

        # Set request data
        $request_data = array(
            'func' => 'trustGetOrderInfo',
            'data' => array(
                'attributes' => array(
                    'order_id' => $order_id,
                )
            )
        );

        try {
            # Execute get SSL details request
            $response = $opensrsLib->process('array', $request_data);
        } catch (\opensrs\APIException $e) {
            # Return exception
            $error_msg = array(
                'is_success' => False,
                'error' => $e->getMessage()
            );
            return $error_msg;
        }

        # Module log
        self::ModuleLog(__FUNCTION__, $request_data, $response->resultFullRaw, '', '');

        if($response->resultFullRaw['is_success'] == True) {
            return $response->resultFullRaw;
        } else {
            return False;
        }
    }

    /**
     * Custom Module logs
     * @param string $function::Function name
     * @param array $request_string::Request data
     * @param array $response_data::Response data
     * @param $processed_data::Post processed data
     * @param array $params::Data from WHMCS
     */
    public function ModuleLog($function, $request_data, $response_data, $processed_data, $params) {
        # Set module name
        $module = 'OnlineNIC_SSL';
        # Construct 'action' name
        $action = substr($function, 0, 11) == 'opensrs_ssl_' ? substr($function, 11) : $function;
        # Set replace conditions for sensitive data (reseller ID and API keys)
        $replacevars = array($params["configoption1"], $params["configoption2"], $params["configoption4"]);
        # Log data
        logModuleCall($module, $action, $request_data, $response_data, $processed_data, $replacevars);
    }
}

?>
